.maps-popover {
    padding: 5px;
}

#dragged-entry {
    color: #939695;
}

.layer-radio-button {
    background-image: none;
    padding: 0;
    border: 1px solid @insensitive_borders;
    outline-width: 1px;
}

.layer-radio-button:checked {
    border: 2px solid @suggested_action_button_a;
}

#zoom-in-button,
#zoom-out-button {
    font-weight: bold;
}

#sidebar-entry-list,
#sidebar-entry-list .list-row:hover {
    background-color: transparent;
}

#sidebar-entry-list .list-row:focus {
    outline: 0;
}

#sidebar-entry-list > GtkLabel {
    padding-right: 6px;
}

#sidebar-entry-list > GtkLabel:dir(rtl) {
    padding-left: 6px;
}

#sidebar-entry-list .transportation-mode-button > GtkImage {
    padding: 4px 7px;
}

#sidebar {
    border-left: 1px solid gray;
}

#sidebar:dir(rtl) {
    border-right: 1px solid gray;
}

#instruction-list {
    border-right: 0px;
    border-left: 0px;
}

#instruction-box > GtkImage {
    padding-right: 6px;
    padding-left: 6px;
}

.bubble-title {
    font-size: large;
    font-weight: bold;
}

.small-label,
.bubble-content,
.bubble-subtitle {
    font-size: small;
}

.maps-check-in GtkTextView {
    font-size: large;
}

.printing-text {
    color: rgb(0, 0, 0);
}

.shaded {
    background-color: alpha(black, 0.1);
    border-bottom: 1px solid alpha(black, 0.2);
}

.route-label {
    padding-left: 2px;
    padding-right: 2px;
    padding-top: 2px;
    padding-bottom: 2px;
    min-width: 16px;
    min-height: 16px;
    font-size: smaller;
}

.small-circular {
  border-radius: 14px;
  min-width: 14px;
  min-height: 14px;
  padding: 0px 0px;
  background-color: transparent;
  -gtk-outline-radius: 14px;
}

